package org.processmining.logdecomposer.filters;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.processmining.logdecomposer.filters.impl.DecompositionInFilter;
import org.processmining.logdecomposer.filters.impl.DecompositionOutFilter;
import org.processmining.logdecomposer.filters.impl.DecompositionRenameFilter;

public class DecompositionFilterManager {

	private static DecompositionFilterManager instance = null;
	private List<DecompositionFilter> miners;
	private DecompositionFilter defaultMiner;
	private boolean sorted;

	private DecompositionFilterManager() {
		miners = new ArrayList<DecompositionFilter>();
		defaultMiner = null;
		sorted = false;
		register(new DecompositionInFilter(), true);
		register(new DecompositionOutFilter(), false);
		register(new DecompositionRenameFilter(), false);
	}

	public static DecompositionFilterManager getInstance() {
		if (instance == null) {
			instance = new DecompositionFilterManager();
		}
		return instance;
	}

	public void register(DecompositionFilter miner, boolean isDefault) {
		miners.add(miner);
		if (isDefault) {
			defaultMiner = miner;
		}
		sorted = false;
	}

	public List<DecompositionFilter> getMiners() {
		if (!sorted) {
			Collections.sort(miners, new Comparator<DecompositionFilter>() {
				public int compare(DecompositionFilter m1, DecompositionFilter m2) {
					return m1.getName().compareTo(m2.getName());
				}
			});
			sorted = true;
		}
		return miners;
	}

	public boolean isDefault(DecompositionFilter miner) {
		return miner == defaultMiner;
	}

	public DecompositionFilter getMiner(String name) {
		if (name != null) {
			for (DecompositionFilter miner : miners) {
				if (name.equals(miner.getName())) {
					return miner;
				}
			}
		}
		return defaultMiner;
	}
}
