package org.processmining.logdecomposer.filters.impl;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.basicutils.utils.StringConstants;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.log.models.EventLogArray;
import org.processmining.log.models.impl.EventLogArrayFactory;
import org.processmining.logdecomposer.classifiers.StartEndClassifier;
import org.processmining.logdecomposer.extensions.XClassExtension;
import org.processmining.logdecomposer.filters.DecompositionFilter;
import org.processmining.logdecomposer.parameters.DecomposeEventLogUsingActivityClusterArrayParameters;

public class DecompositionInFilter implements DecompositionFilter {

	public final static String NAME = "Filter In";
	
	public EventLogArray apply(PluginContext context, XLog eventLog, ActivityClusterArray clusters,
			DecomposeEventLogUsingActivityClusterArrayParameters parameters) {
		//Create the subLogs object
		EventLogArray eventLogs = EventLogArrayFactory.createEventLogArray();
		eventLogs.init();
		
		XEventClassifier artificialClassifier = new StartEndClassifier(parameters.getClassifier());

		XFactory factory = XFactoryRegistry.instance().currentDefault();
		//Compute the labels of each cluster and create an empty sublog for each cluster
		Map<Integer, Set<String>> labelsMap = new HashMap<Integer, Set<String>>();
		for (Set<XEventClass> activities : clusters.getClusters()) {
			Set<String> labels = new HashSet<String>();
			for (XEventClass activity : activities) {
				labels.add(activity.getId());
			}
			XLog subLog = factory.createLog((XAttributeMap) eventLog.getAttributes().clone());
			if (parameters.isAddStartEndEvents()) {
				subLog.getClassifiers().add(artificialClassifier);
			}
			subLog.getClassifiers().addAll(eventLog.getClassifiers());
			subLog.getExtensions().addAll(eventLog.getExtensions());
			subLog.getGlobalTraceAttributes().addAll(eventLog.getGlobalTraceAttributes());
			subLog.getGlobalEventAttributes().addAll(eventLog.getGlobalEventAttributes());
			int idx = eventLogs.addLog(subLog);
			labelsMap.put(idx, labels);
		}

		//Walk thought each trace of the original log
		Map<Integer, XTrace> tracesMap = new HashMap<Integer, XTrace>();
		Iterator<XTrace> itTrace = eventLog.iterator();
		int num = 0;
		while (itTrace.hasNext()) {
			XTrace trace = itTrace.next();

			//Create a empty trace for each cluster
			for (Integer i : labelsMap.keySet()) {
				XTrace subTrace = factory.createTrace((XAttributeMap) trace.getAttributes().clone());
				tracesMap.put(i, subTrace);
			}

			// Add optional start events
			if (parameters.isAddStartEndEvents()) {
				for (Integer i : labelsMap.keySet()) {
					XEvent startEvent = factory.createEvent();
					XClassExtension.instance().assignEventId(startEvent, StringConstants.START);
					tracesMap.get(i).add(startEvent);
				}
			}

			//Walk thought each event of the original trace
			Iterator<XEvent> itEvent = trace.iterator();
			while (itEvent.hasNext()) {
				XEvent event = itEvent.next();
				String id = parameters.getClassifier().getClassIdentity(event);

				for (Entry<Integer, Set<String>> labelsEntry : labelsMap.entrySet()) {
					if (labelsEntry.getValue().contains(id)) {
						XEvent subEvent = factory.createEvent((XAttributeMap) event.getAttributes().clone());
						tracesMap.get(labelsEntry.getKey()).add(subEvent);
					}
				}
			}

			// Add optional end events
			if (parameters.isAddStartEndEvents()) {
				for (Integer i : labelsMap.keySet()) {
					XEvent endEvent = factory.createEvent();
					XClassExtension.instance().assignEventId(endEvent, StringConstants.END);
					tracesMap.get(i).add(endEvent);
				}
			}

			//Add the traces to each sublogs of each cluster
			for (Integer i : tracesMap.keySet()) {
				XTrace subTrace = tracesMap.get(i);
				if (!(parameters.isRemoveEmptyTraces() && subTrace.isEmpty())) {
					eventLogs.getLog(i).add(subTrace);
				}

			}

			num++;
			if (num % 100 == 0) {
//				context.log("Trace num " + num);
			}
		}

		return eventLogs;
	}

	public String getName() {
		return NAME;
	}

}
