package org.processmining.logdecomposer.plugins;

import java.util.Collection;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.activityclusterarray.models.ActivityClusterArray;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.log.models.EventLogArray;
import org.processmining.logdecomposer.algorithms.DecomposeEventLogUsingActivityClusterArrayAlgorithm;
import org.processmining.logdecomposer.connections.DecomposeEventLogUsingActivityClusterArrayConnection;
import org.processmining.logdecomposer.dialogs.DecomposeEventLogUsingActivityClusterArrayDialog;
import org.processmining.logdecomposer.help.DecomposeEventLogUsingActivityClusterArrayHelp;
import org.processmining.logdecomposer.parameters.DecomposeEventLogUsingActivityClusterArrayParameters;

@Plugin(name = "Split Event Log", parameterLabels = { "Event Log", "Activity Cluster Array", "Parameters" }, returnLabels = { "Event Log Array" }, returnTypes = { EventLogArray.class }, help = DecomposeEventLogUsingActivityClusterArrayHelp.TEXT)
public class DecomposeEventLogUsingActivityClusterArrayPlugin extends DecomposeEventLogUsingActivityClusterArrayAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Split Event Log, UI", requiredParameterLabels = { 0, 1 })
	public EventLogArray runUI(UIPluginContext context, XLog eventLog, ActivityClusterArray clusters) {
		DecomposeEventLogUsingActivityClusterArrayParameters parameters = new DecomposeEventLogUsingActivityClusterArrayParameters(
				eventLog);
		DecomposeEventLogUsingActivityClusterArrayDialog dialog = new DecomposeEventLogUsingActivityClusterArrayDialog(
				eventLog, parameters);
		InteractionResult result = context.showWizard("Configure split (classifier, empty traces)", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			return null;
		}
		return decomposePrivateConnection(context, eventLog, clusters, parameters);
	}

	/**
	 * @deprecated Use runUI() instead
	 */
	@Deprecated
	public EventLogArray decomposeUI(UIPluginContext context, XLog eventLog, ActivityClusterArray clusters) {
		return runUI(context, eventLog, clusters);
	}

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Split Event Log, Default", requiredParameterLabels = { 0, 1 })
	public EventLogArray runDefault(PluginContext context, XLog eventLog, ActivityClusterArray clusters) {
		DecomposeEventLogUsingActivityClusterArrayParameters parameters = new DecomposeEventLogUsingActivityClusterArrayParameters(
				eventLog);
		return decomposePrivateConnection(context, eventLog, clusters, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead
	 */
	@Deprecated
	public EventLogArray decomposeDefault(PluginContext context, XLog eventLog, ActivityClusterArray clusters) {
		return runDefault(context, eventLog, clusters);
	}

	@PluginVariant(variantLabel = "Split Event Log, Parameters", requiredParameterLabels = { 0, 1, 2 })
	public EventLogArray run(PluginContext context, XLog eventLog, ActivityClusterArray clusters,
			DecomposeEventLogUsingActivityClusterArrayParameters parameters) {
		return decomposePrivateConnection(context, eventLog, clusters, parameters);
	}

	/**
	 * @deprecated Use run() instead
	 */
	@Deprecated
	public EventLogArray decomposeParameters(PluginContext context, XLog eventLog, ActivityClusterArray clusters,
			DecomposeEventLogUsingActivityClusterArrayParameters parameters) {
		return run(context, eventLog, clusters, parameters);
	}

	private EventLogArray decomposePrivateConnection(PluginContext context, XLog eventLog,
			ActivityClusterArray clusters, DecomposeEventLogUsingActivityClusterArrayParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<DecomposeEventLogUsingActivityClusterArrayConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(
						DecomposeEventLogUsingActivityClusterArrayConnection.class, context, eventLog);
				for (DecomposeEventLogUsingActivityClusterArrayConnection connection : connections) {
					if (connection.getObjectWithRole(DecomposeEventLogUsingActivityClusterArrayConnection.LOG).equals(
							eventLog)
							&& connection.getObjectWithRole(
									DecomposeEventLogUsingActivityClusterArrayConnection.CLUSTERS).equals(clusters)
							&& connection.getParameters().equals(parameters)) {
						return connection.getObjectWithRole(DecomposeEventLogUsingActivityClusterArrayConnection.LOGS);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		EventLogArray eventLogs = apply(context, eventLog, clusters, parameters);
		if (parameters.isTryConnections()) {
			context.getConnectionManager()
					.addConnection(
							new DecomposeEventLogUsingActivityClusterArrayConnection(eventLog, clusters, eventLogs,
									parameters));
		}
		return eventLogs;
	}

}
